#Whole Life-Cycle-Analysis WS 22/23
#Group Project Assignment
#GROUP E

#Mina Astan (387413)
#RAMI MOHAMED ALI (474547)
#Sangaj Raghunath Haladkar (485120)
#Christoph Kirsch (321322)
#Ahmad Sabha (0480306)

rm (list = ls())

library(timelineS)
library(dplyr)
library(MCDA) 
library(ggplot2)
library(scales)
library(reshape2)
library(tibble)
library(lubridate)
library(readxl)
library(mco)
library(MASS)
library(tidyr)
library(stringr)
library(rPref)


# MAINTENANCE STRATEGIES (AHMAD) ----------------------------------------------------------------------

plot.timeline <- function(lifetime, events.name, start.Date, plot.Name) {
  dataP <- data.frame(Events = events.name, Event_Dates = ymd(start.Date) + years(lifetime))
  timelineS(dataP, main = plot.Name, labels = events.name, label.direction = "up", label.position = 3)
}

dist.Events <- function(lifetime, events, start.Date, option.Name) {
  events <- sort(events, decreasing = TRUE)
  distribution.events <- sapply(events, seq, from = 0, to = lifetime)
  # all events unlisted
  all.events <- melt(distribution.events)
  colnames(all.events) <- c("frequency", "event")
  
  # sort the events ascending
  all.events <- all.events[order(all.events$frequency),]
  
  # get the unique sequence of events
  unique.events <- all.events[!duplicated(all.events$frequency),]
  unique.events$event[which(unique.events$frequency == 0)] <- "DC"
  
  # plot.timeline(unique.events$frequency, unique.events$event, start.Date, option.Name)
  return(unique.events)
}


#Combination Function -------------

combine.lifeTimelines <- function(product.1, product.2, product.3, product.4, product.5, p1.dur, p2.dur, p3.dur, p4.dur, p5.dur) {
  
  base <- list(Names = c(), frequency = c(), duration = c())
  
  base$frequency <- sort(unique(c(product.1$frequency, product.2$frequency, product.3$frequency, product.4$frequency, product.5$frequency)), decreasing = FALSE)
  base$Names[1] <- "DC"
  base$duration[1] <- 0
  base$Names[length(base$frequency)] <- "END"
  base$duration[length(base$frequency)] <- 0
  
  for(index in 2:(length(base$frequency) - 1)) {
    
    L1 <- length(phase.1 <- product.1$event[which(product.1$frequency == base$frequency[index])])
    L2 <- length(phase.2 <- product.2$event[which(product.2$frequency == base$frequency[index])])
    L3 <- length(phase.3 <- product.3$event[which(product.3$frequency == base$frequency[index])])
    L4 <- length(phase.4 <- product.4$event[which(product.4$frequency == base$frequency[index])])
    L5 <- length(phase.5 <- product.5$event[which(product.5$frequency == base$frequency[index])])
    
    cond <- c(L1, L2, L3, L4, L5)
    c <- 1
    durations <- c()
    
    for (p in cond) {
      if (p != 0) {
        base$Names[index] <- ifelse(is.na(base$Names[index]), get(paste0("phase.", c)), str_split(paste(base$Names[index], get(paste0("phase.", c))), " ", simplify = TRUE))
        durations <- append(durations, get(paste0("p", c, ".dur"))[which(names(get(paste0("p", c, ".dur"))) == get(paste0("phase.", c)))], after = length(durations))
        base$duration[index] <- ifelse(length(durations) == 0, durations, max(durations))
      }
      c <- c+1
    }
    
    if (length(durations) == 0) {
      base$duration[index] <- 0
    }
    
  }
  
  return(base)
}


par(mfrow = c(2,1))
start.Date = "2023-03-01"
lifetime <- 100

events.Op5.bTimber = c(M.t = 3, DR = 10, END = lifetime)
events.Op3.bSteel = c(M.s = 10, PSR = 45, END = lifetime)
events.Op4.Building = c(M.b = 5, PCR = 45, END = lifetime)
events.Op2.PV = c(M.pv = 10, END = lifetime)
events.Op1.Pavement = c(MO = 3, M.p = 5, PR = 15, END = lifetime)

duration.Op5.bTimber <- c(M.t = 3, DR = 25)
duration.Op3.bSteel <- c(M.s = 2, PSR = 30)
duration.Op4.Building <- c(M.b = 5, PCR = 50)
duration.Op2.PV <- c(M.pv = 2, Dummy = 0)
duration.Op1.Pavement <- c(MO = 2, M.p =5, PR = 20)

design.Options.bTimber <- list(design.Op5 <- "Timber Bridge: Stress-Laminated Sawn Timber Deck")
design.Options.bSteel <- list(design.Op3 <- "Steel Bridge: Fiber reinforced polymer (FRP) decking")
design.Options.Building <- list(design.Op4 <- "Building: Prefab Beam and steel columns")
design.Options.PV <- list(design.Op2 <- "PV Panels: Li-Ion")
design.Options.Pavement <- list(design.Op1 <- "Airport Pavement: Asphalt(4) surface and P-208(15) Base layer")

maintenance.bTimber <- as.data.frame(dist.Events(lifetime, events.Op5.bTimber, start.Date, design.Options.bTimber))
maintenance.bSteel <- as.data.frame(dist.Events(lifetime, events.Op3.bSteel, start.Date, design.Options.bSteel))
maintenance.Building <- as.data.frame(dist.Events(lifetime, events.Op4.Building, start.Date, design.Options.Building))
maintenance.PV <- as.data.frame(dist.Events(lifetime, events.Op2.PV, start.Date, design.Options.PV))
maintenance.Pavement <- as.data.frame(dist.Events(lifetime, events.Op1.Pavement, start.Date, design.Options.Pavement))

integrated.interv <- combine.lifeTimelines(maintenance.bTimber, maintenance.bSteel, maintenance.Building, maintenance.PV, maintenance.Pavement,
                                           duration.Op5.bTimber, duration.Op3.bSteel, duration.Op4.Building, duration.Op2.PV, duration.Op1.Pavement)

sum(integrated.interv$duration)


#Scienarios Automation Function ---------------

design.explore <- function(events1, events2, events3, events4, events5) {
  results <- c()
  for(i in 1: dim(events1)[1]) {
    ev1 <- unlist(events1[i, ])
    dist.1 <- as.data.frame(dist.Events(lifetime, ev1, start.Date, design.Op5))
    # dur.ev1 <- ev1/2
    for (j in 1: dim(events2)[1]) {
      ev2 <- unlist(events2[j, ])
      dist.2 <- as.data.frame(dist.Events(lifetime, ev2, start.Date, design.Op3))
      # dur.ev2 <- ev2/2
      for(k in 1: dim(events3)[1]) {
        ev3 <- unlist(events3[k, ])
        dist.3 <- as.data.frame(dist.Events(lifetime, ev3, start.Date, design.O4))
        # dur.ev3 <- ev3/2
        for(l in 1: dim(events4)[1]) {
          ev4 <- unlist(events4[l, ])
          dist.4 <- as.data.frame(dist.Events(lifetime, ev4, start.Date, design.Op2))
          # dur.ev4 <- ev4/2
          updated.duration.Op2.PV <- c(M.pv = 2, Dummy = 0)
          for(n in 1: dim(events5)[1]) {
            ev5 <- unlist(events5[n, ])
            dist.5 <- as.data.frame(dist.Events(lifetime, ev5, start.Date, design.Op1))
            # dur.ev5 <- ev5/2
            combined.lifetime <- combine.lifeTimelines(dist.1, dist.2, dist.3, dist.4, dist.5,
                                                       duration.Op5.bTimber, duration.Op3.bSteel, duration.Op4.Building, duration.Op2.PV, duration.Op1.Pavement)
            min.dist.int <- min(abs(combined.lifetime$frequency[1:(length(combined.lifetime$frequency) - 1)] -
                                      combined.lifetime$frequency[2:length(combined.lifetime$frequency)]))
            results <- rbind(results, c(ev1, ev2, ev3, ev4, ev5, dur = sum(combined.lifetime$duration), dist.inter = min.dist.int))
          }
        }
      }
    }
  }
  return(as.data.frame(results))
}


#Scenarios Definition-----------------

n.grid <- 3

events.grid.bTimber <- expand.grid(M.t = sample(seq(3,8, by = 1), n.grid),
                                   DR = sample(seq(8,15,1), n.grid))

events.grid.bSteel <- expand.grid(M.s = sample(seq(7,12, by = 1), n.grid),
                                  PSR = sample(seq(40,50,1), n.grid))

events.grid.Building <- expand.grid(M.b = sample(seq(3,8, by = 1), n.grid),
                                    PCR = sample(seq(40,50,1), n.grid))

events.grid.PV <- expand.grid(M.pv = sample(seq(3,8, by = 1), n.grid), Dummy = lifetime+1)

events.grid.Pavement <- expand.grid(MO = sample(seq(3,8, by = 1), n.grid),
                                    M.p = sample(seq(3,8,1), n.grid),
                                    PR = sample(seq(10,20,1), n.grid))


# WARNING: RUNNING "response.space" FUNCTION BELOW WILL TAKE AT LEAST 15 MINS!

# response.space <- design.explore(events.grid.bTimber, events.grid.bSteel, events.grid.Building, events.grid.PV, events.grid.Pavement)
# 
# 
# #Results Visualization ---------------
# 
# p <- low(dur)* high(dist.inter)
# sky <- psel(response.space, p)
# pareto2 <- psel(response.space, p, top = nrow(response.space))
# 
# ggplot(response.space, aes(x = dur, y = dist.inter)) +
#   geom_point(shape = 21) +
#   geom_point(data = pareto2, size = 3, aes(color = factor(pareto2$.level)))
# 
# show_front <- function(pref) {
#   plot(response.space$dur, response.space$dist.inter)
#   sky <- psel(response.space, pref)
#   plot_front(response.space, pref, col = rgb(0, 0, 1))
#   points(sky$dur, sky$dist.inter, lwd = 3)
# }
# 
# show_front(p)
# 
# p <- high(dur) * low(dist.inter)
# show_front(p)




# LCI (RAMI) ----------------------------------------------------------------------


LCI.materials = read.csv(file = "LCI.csv")

#Rami

LCA.pave <- function(length, width, surface.Option, base.Option, materials, dist.event)
  
{ 
  materials.split <- split(materials, materials$scope)
  # calculate the volume of the deck based on different materials strategies
  
  #new line to get the interventions
  sp <- summary(as.factor(dist.event$event))
  sp2 <- as.data.frame(rbind(Freq = sp), stringsAsFactors=F, row.names = 1:length(sp))
  
  if(surface.Option == "asphalt_4")
  {
    surf.volume <- length * width * 0.1016
    interventions.sur <- sample(8:15, size=1) #sp2$DC + 0.35*sp2$M.p + 0.65*sp2$PR - NOT WORKING!
    
  }
  if(base.Option == "P_208_15")
  {
    base.volume <- length * width * 0.381
    interventions.base <- interventions.sur #sp2$DC + 0.35*sp2$M.p + 0.65*sp2$PR - NOT WORKING!
  }
  
  surface <- mutate(materials.split$asphalt_4, pave.Q = surf.volume, interventions = interventions.sur) 
  base <- mutate(materials.split$P_208_15, pave.Q = base.volume, interventions = interventions.base)
  
  LCA.matrix1 <- rbind(surface, base)
  
  LCA.matrix1 <- mutate(LCA.matrix1, TotalMaterials.Q1 = quantities * pave.Q / 1000,
                        materials.LC = TotalMaterials.Q1 * interventions,
                        Energy.LC1 = materials.LC * energy, 
                        CO2.LC1 = materials.LC * CO2 * 1000,
                        NOX.LC1 = materials.LC * NOX * 1000,
                        SO2.LC1 = materials.LC * SO2 * 1000)
  LCA.result1 <- list(Energy1 = sum(LCA.matrix1$Energy.LC1),
                      CO21 = sum(LCA.matrix1$CO2.LC1),
                      NOX1 = sum(LCA.matrix1$NOX.LC1),
                      SO21 = sum(LCA.matrix1$SO2.LC1))
  
}

#cris

LCA.battery <- function(battery.Option, materials, dist.event) 
{
  
  #create subsets of materials
  materials.split <- split(materials, materials$scope)
  
  # calculate the mass of the battery of based on different specific energy 
  
  #new line to get the interventions
  spv <- summary(as.factor(dist.event$event))
  spv2 <- as.data.frame(rbind(Freq = spv), stringsAsFactors=F, row.names = 1:length(spv))
  
  if (battery.Option == "LiIon") { 
    battery.mass <-  4000/120
    replacement.factor <- sample(8:15, size=1) #spv2$DC + 0.5*spv2$M.pv - NOT WORKING
  } 
  
  #battery.Options <- c("LiIon") 
  solar <- mutate(materials.split$LiIon, battery.Q = battery.mass, interventions = replacement.factor)
  
  LCA.matrix2 <- rbind(solar)
  
  LCA.matrix2 <- mutate(LCA.matrix2, TotalMaterials.Q2 = quantities * battery.Q,
                        battery.LC = TotalMaterials.Q2 * interventions,
                        Energy.LC2 = battery.LC * energy,
                        CO2.LC2 = battery.LC * CO2,
                        NOX.LC2 = battery.LC * NOX / 1000,
                        SO2.LC2 = battery.LC * SO2 / 1000)
  LCA.result2 <- list(Energy2 = sum(LCA.matrix2$Energy.LC2),
                      CO22 = sum(LCA.matrix2$CO2.LC2),
                      NOX2 = sum(LCA.matrix2$NOX.LC2),
                      SO22 = sum(LCA.matrix2$SO2.LC2))
}
#SANGAJ

LCA.building <- function(length, width, height, column.Option, beam.Option, 
                         materials, dist.event)
{
  
  prefab.column.Section <- 0.30
  steel.column.unitWeight <- 500 #the weight for A36 steel channel
  materials.split <- split(materials, materials$scope)
  # calculate the volume of the beam based on different materials strategies
  
  #new line to get the interventions
  sb <- summary(as.factor(dist.event$event))
  sb2 <- as.data.frame(rbind(Freq = sb), stringsAsFactors=F, row.names = 1:length(sb))
  
  if (beam.Option == "PRC") {
    beam.volume <-  length * width * height
    interventions.beam <- sb2$DC + 0.60*sb2$PCR }
  #column options
  
  if (column.Option == "steel") {
    n <- round(width / 3, 0)
    column.V <- n * steel.column.unitWeight * length
    interventions.column <- sb2$DC + 0.60*sb2$PCR
  } 
  
  beam <- mutate(materials.split$PRC, building.Q = beam.volume, interventions = 
                   interventions.beam)
  
  column <- mutate(materials.split$steel, building.Q = column.V,
                   interventions = interventions.column)
  LCA.matrix3 <- rbind(beam, column)
  
  LCA.matrix3 <- mutate(LCA.matrix3, TotalMaterials.Q3 = quantities * building.Q / 1000,
                        materials.LC3 = TotalMaterials.Q3 * interventions,
                        Energy.LC3 = materials.LC3 * energy, 
                        CO2.LC3 = materials.LC3 * CO2 * 1000,
                        NOX.LC3 = materials.LC3 * NOX * 1000,
                        SO2.LC3 = materials.LC3 * SO2 * 1000)
  LCA.result3 <- list(Energy3 = sum(LCA.matrix3$Energy.LC3),
                      CO23 = sum(LCA.matrix3$CO2.LC3),
                      NOX3 = sum(LCA.matrix3$NOX.LC3),
                      SO23 = sum(LCA.matrix3$SO2.LC3))
}

#AHMAD   
LCA.bridge1 <- function(length, width, height, thickness, girder.Option, deck1.Option, materials, dist.event) 
{  
  glulam.girder.Section <- 0.35 #m2
  sawn.girder.Section <- 0.2 #m2
  steel.bar.section <- 0.000254 #m2
  steel.bar.weight <- 2 #kg/m
  asphalt.Q <- length * width * thickness #m3
  
  materials.split <- split(materials, materials$scope)
  
  # calculate the volume of the deck based on different materials strategies
  
  #new line to get the interventions
  sbt <- summary(as.factor(dist.event$event))
  sbt2 <- as.data.frame(rbind(Freq = sbt), stringsAsFactors=F, row.names = 1:length(sbt))
  
  if(deck1.Option == "SLT") {
    deck.volume <- length * width * height
    #get the number of tendons across the width (spacing = 0.84m)
    n <- round(width / 0.84, 0)
    steel.anchor.V <- n * steel.bar.weight * length
    interventions.deck <- sbt2$DC + sbt2$DR
  } 
  
  # girder options
  
  if (girder.Option == "ST") {
    n <- round(width / 3, 0)
    interventions.girders <- sbt2$DC + sbt2$DR
    girders.V <- n * sawn.girder.Section * length
  } 
  
  asphalt <- mutate(materials.split$asphalt, bridge1.Q = asphalt.Q, interventions = sbt2$DC + 0.5*sbt2$M.t)
  deck <- mutate(materials.split$SLT, bridge1.Q = deck.volume, interventions = interventions.deck)
  girders <- mutate(materials.split$ST, bridge1.Q = girders.V, interventions = interventions.girders)
  
  LCA.matrix4 <- rbind(asphalt, deck, girders)
  LCA.matrix4 <- mutate(LCA.matrix4, TotalMaterials.Q = quantities * bridge1.Q,
                       materials.LC4 = TotalMaterials.Q * interventions,
                       Energy.LC4 = materials.LC4 * energy,
                       CO2.LC4 = materials.LC4 * CO2,
                       NOX.LC4 = materials.LC4 * NOX,
                       SO2.LC4 = materials.LC4 * SO2)
  LCA.result4 <- list(Energy4 = sum(LCA.matrix4$Energy.LC4),
                      CO24 = sum(LCA.matrix4$CO2.LC4),
                      NOX4 = sum(LCA.matrix4$NOX.LC4),
                      SO24 = sum(LCA.matrix4$SO2.LC4))
} 



#MINA   

LCA.bridge2 <- function(length, width, height, thickness, deck2.Option, materials, dist.event) 
  
  
{ 
  materials.split <- split(materials, materials$scope)
  prefab.Section <- 0.78
  steel.unitWeight <- 317 #the weight for HEM800 steel profile KG/M
  ribs.tk <- 0.004
  asphalt.tk <- 0.12
  steel.plate.tk <- 0.02
  length <- 25
  width <- 25
  height <- 0.4
  steel.plate.v <- length * width * steel.plate.tk
  asphalt1.Q <- length * width * asphalt.tk
  
  
  
  
  
  #_________________________________ deck options
  
  #new line to get the interventions
  sbs <- summary(as.factor(dist.event$event))
  sbs2 <- as.data.frame(rbind(Freq = sbs), stringsAsFactors=F, row.names = 1:length(sbs))
  
  if (deck2.Option == "FRP") {
    
    deck.volume1 <- 5* length * width * height
    interventions.deck1 <- sbs2$DC + 0.35*sbs2$PSR
    n1 <- round(width/2.5)
    
    beam.width <- 0.04
    beam.height <- 0.15
    beam.v <- 3*n1* length * beam.width * beam.height*prefab.Section
    #get the volume of whole used steel:
    steel.v <- steel.plate.v +beam.v
    interventions.steel <- sbs2$DC + 0.55*sbs2$PSR
    deck.volume1 <- deck.volume1+steel.v 
  }
  asphalt1 <- mutate(materials.split$asphalt, bridge2.Q = asphalt1.Q, interventions = sbs2$DC + 0.5*sbs2$M.s)   
  deck1 <- mutate(materials.split$FRP, bridge2.Q = deck.volume1, interventions = interventions.deck1)
  steel <- mutate(materials.split$steel, bridge2.Q = steel.v, interventions = interventions.steel)
  
  LCA.matrix5 <- rbind(asphalt1, deck1, steel)
  LCA.matrix5 <- mutate(LCA.matrix5, TotalMaterials.Q5 = quantities * bridge2.Q / 1000,
                        materials.LC5 = TotalMaterials.Q5 * interventions,
                        Energy.LC5 = materials.LC5 * energy, 
                        CO2.LC5 = materials.LC5 * CO2 * 1000,
                        NOX.LC5 = materials.LC5 * NOX * 1000,
                        SO2.LC5 = materials.LC5 * SO2 * 1000)
  LCA.result5 <- list(Energy5 = sum(LCA.matrix5$Energy.LC5),
                      CO25 = sum(LCA.matrix5$CO2.LC5),
                      NOX5 = sum(LCA.matrix5$NOX.LC5),
                      SO25 = sum(LCA.matrix5$SO2.LC5))
}


p.length <- 4000 # units: m 
p.width <- 45 #units m

bt.length <- 20
bt.width <- 15
bdt.depth <- 0.3

bs.length    <- 25 # units: m
bs.width     <- 20 #units m 
bs.height <- 0.4 + 0.02 #units m 
asphalt.tk  <- 0.12 #units m 

d.length <- 3 # units: m
d.width <- 0.23 #units m
d.depth <- 0.38 #units m



deck1.options <- c("SLT")
deck2.options <- c("FRP")
base.options <- c("P_208_15")
surface.options <- c("asphalt_4")
battery.options <- c("LiIon")
girder.options <- c("ST")
column.options <- c("steel")
beam.options <- c("PRC")


TimberBridge <- LCA.bridge1(bt.length, bt.width, bdt.depth, asphalt.tk, girder.options[1], deck1.options[1], LCI.materials, maintenance.bTimber)
SteelBridge <- LCA.bridge2(bs.length, bs.width, bs.height, asphalt.tk,  deck2.options[1], LCI.materials, maintenance.bSteel)
AirPavement <- LCA.pave(p.length, p.width, surface.options[1], base.options[1], LCI.materials, maintenance.Pavement)
Building <- LCA.building(d.length, d.width, d.depth, column.options[1], beam.options[1], LCI.materials, maintenance.Building)
Battery <- LCA.battery(battery.options[1], LCI.materials, maintenance.PV)

integrated.Design <- as.data.frame(list(Energy = TimberBridge$Energy4 + SteelBridge$Energy5 + AirPavement$Energy1 +
                                          Building$Energy3 + Battery$Energy2,
                                        CO2 = TimberBridge$CO24 + SteelBridge$CO25 + AirPavement$CO21 +
                                          Building$CO23 + Battery$CO22,
                                        NOx = TimberBridge$NOX4 + SteelBridge$NOX5 + AirPavement$NOX1 +
                                          Building$NOX3+ Battery$NOX2,
                                        SO2 = TimberBridge$SO24 + SteelBridge$SO25 + AirPavement$SO21 +
                                          Building$SO23 + Battery$SO22))


Energy.costs <- 0.128
CO2.unitcost <- 26 # per metric tone
NOx.unitCost <- 42 # per metric tone
SO2.unitCosts <- 85 # per metric tone

integrated.Design <- mutate(integrated.Design,
                            Costs = (Energy * Energy.costs +
                                       CO2*CO2.unitcost +
                                       NOx * NOx.unitCost +
                                       SO2 * SO2.unitCosts)/10^9)

# MULTI OBJECTIVE OPTIMIZATION (SANGAJ) ----------------------------------------------------------------------

fitness <- function(x) {
  #define the output dimension
  z <- numeric(7)
  x <- round(x, 0)
  y <- expand.grid(M.t = x[1], DR = x[2], M.s = x[3], PSR = x[4], M.b = x[5], PCR = x[6], M.pv = x[7], MO = x[8], M.p= x[9], PR = x[10])
  dur.ev1 <- duration.Op5.bTimber
  dur.ev2 <- duration.Op3.bSteel
  dur.ev3 <- duration.Op4.Building
  dur.ev4 <- duration.Op2.PV
  dur.ev5 <- duration.Op1.Pavement
  dist.1 <- apply(y[1:2], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  dist.2 <- apply(y[3:4], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  dist.3 <- apply(y[5:6], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  dist.4 <- apply(y[7], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  dist.5 <- apply(y[8:10], 1, FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  results <- combine.lifeTimelines(dist.1[[1]], dist.2[[1]],dist.3[[1]],dist.4[[1]],dist.5[[1]], dur.ev1, dur.ev2, dur.ev3, dur.ev4, dur.ev5)
  z[1] <- sum(results[["duration"]])
  z[2] <- -min(abs(results$frequency[1:(length(results$frequency) - 1)] - results$frequency[2:length(results$frequency)]))
  

  p.length <- 4000 # units: m 
  p.width <- 45 #units m
  
  bt.length <- 20
  bt.width <- x[11]
  bdt.depth <- 0.3
  
  bs.length    <- 25 # units: m
  bs.width     <- 20 #units m 
  bs.height <- 0.4 + 0.02 #units m 
  asphalt.tk  <- 0.12 #units m 
  
  d.length <- 3 # units: m
  d.width <- 0.23 #units m
  d.depth <- 0.38 #units m
  materials <- LCI.materials
  deck1.options <- c("SLT")
  deck2.options <- c("FRP")
  base.options <- c("P_208_15")
  surface.options <- c("asphalt_4")
  battery.options <- c("LiIon")
  girder.options <- c("ST")
  column.options <- c("steel")
  beam.options <- c("PRC")
  
  Energy.costs <- 0.128
  CO2.unitcost <- 26 # per metric tone
  NOx.unitCost <- 42 # per metric tone
  SO2.unitCosts <- 85 # per metric tone
  
  product.1.TimberBridge <- LCA.bridge1(bt.length, bt.width, bdt.depth, asphalt.tk, girder.options[1], deck1.options[1], LCI.materials, dist.1[[1]])
  product.2.SteelBridge <- LCA.bridge2(bs.length, bs.width, bs.height, asphalt.tk,  deck2.options[1], LCI.materials, dist.2[[1]])
  product.3.AirPavement <- LCA.pave(p.length, p.width, surface.options[1], base.options[1], LCI.materials, dist.5[[1]])
  product.4.Building <- LCA.building(d.length, d.width, d.depth, column.options[1], beam.options[1], LCI.materials, dist.3[[1]])
  product.5.Battery <- LCA.battery(battery.options[1], LCI.materials, dist.4[[1]])
  
  integrated.system <- as.data.frame(list(Energy = product.1.TimberBridge$Energy4
                                          + product.2.SteelBridge$Energy5
                                          + product.3.AirPavement$Energy1
                                          + product.4.Building$Energy3
                                          + product.5.Battery$Energy2,
                                          CO2 = product.1.TimberBridge$CO24
                                          + product.2.SteelBridge$CO25
                                          + product.3.AirPavement$CO21
                                          + product.4.Building$CO23
                                          + product.5.Battery$CO2,
                                          NOx = product.1.TimberBridge$NOX4
                                          + product.2.SteelBridge$NOX5
                                          + product.3.AirPavement$NOX1
                                          + product.4.Building$NOX3
                                          + product.5.Battery$NOX2,
                                          SO2 = product.1.TimberBridge$SO24
                                          + product.2.SteelBridge$SO25
                                          + product.3.AirPavement$SO21
                                          + product.4.Building$SO23
                                          + product.5.Battery$SO22))
  
  integrated.system <- mutate(integrated.system,
                              Costs = (Energy * Energy.costs +
                                         CO2*CO2.unitcost +
                                         NOx * NOx.unitCost +
                                         SO2 * SO2.unitCosts)/10^9) 
  
  z[3] <- integrated.system$Energy
  z[4] <- integrated.system$CO2
  z[5] <- integrated.system$NOx
  z[6] <- integrated.system$SO2
  z[7] <- integrated.system$Costs
  
  return(z)
}

r2 <- nsga2(fitness, idim = 11, odim = 7,
            generations=5, popsize=100,
            lower.bounds=c(3,8,7,40,3,40,3,3,3,10,15),
            upper.bounds= c(8,15,12,50,8,50,8,8,8,20,20))

r2Results <- as.data.frame(r2$value)
outNames <- c("duration", "interv.dist", "energy", "co2", "nox", "so2", "cost" )
colnames(r2Results) <- outNames
pareto3 <- as.data.frame(paretoFront(r2))
colnames(pareto3) <- outNames

ggplot(r2Results, aes(x = duration, y = cost)) + geom_point(shape = 21) +
  geom_point(data = pareto3, size = 3, color="red") +
  geom_line(data = pareto3, color="blue")

ggplot(r2Results, aes(x = duration, y = energy)) + geom_point(shape = 21) +
  geom_point(data = pareto3, size = 3, color="red") +
  geom_line(data = pareto3, color="blue")

ggplot(r2Results, aes(x = duration, y = co2)) + geom_point(shape = 21) +
  geom_point(data = pareto3, size = 3, color="red") +
  geom_line(data = pareto3, color="blue")

input.params <- round(r2$par, 0)
all.results <- cbind(input.params, r2Results, r2$pareto.optimal)
colnames(all.results) <- c("M.t", "DR", "M.s","PSR", 
                           "M.b", "PCR", "M.pv", "MO", "M.p",
                           "PR", "bt.width", "duration", "interv.dist", 
                           "energy", "co2", "nox", "so2", "cost", 
                           "pareto")
par(mfrow = c(1,1))
parcoord(all.results[, 1:18], var.label = T, 
         col = ifelse(all.results$pareto == TRUE, "indianred", "skyblue2"), 
         lty = ifelse(all.results$pareto == TRUE, 1, 3), 
         lwd = ifelse(all.results$pareto == TRUE, 3, 1))